use raylib::{prelude::*, ffi::{GetMouseX, GetMouseY}};





fn find_line_length ( start_x : f32, start_y : f32, end_x : f32, end_y : f32 ) -> f32 {

    let dx = end_x - start_x;
    let dy = end_y - start_y;

    let distance_squared = dx * dx + dy * dy;

    return distance_squared.sqrt();

}


struct Player {
    pos : Vector2,
    size : f32
}

impl Player {

    fn new (x: f32) -> Self {
        Self {
            pos: Vector2 { x: x, y: 500f32 },
            size: 30f32
        }
    }

    fn draw(&self, drawHandle: &mut RaylibDrawHandle) {
        drawHandle.draw_circle(self.pos.x as i32, self.pos.y as i32, self.size, Color::BLUE);
    }

    fn draw_points(&self, draw_handle: &mut RaylibDrawHandle, rects: &Vec<Box>) {

        let num_points = 10000;
        let size = 400f32;

        let mut vertices: Vec<Vector2> = vec![Vector2::default(); num_points];

        for i in 0..num_points {
            let angle = 2.0 * std::f64::consts::PI * (i as f64) / (num_points as f64);
            let x = self.pos.x + self.size * angle.cos() as f32;
            let y = self.pos.y + self.size * angle.sin() as f32;

            let end_x = self.pos.x + size * angle.cos() as f32; // Adjust the factor for desired line length
            let end_y = self.pos.y + size * angle.sin() as f32; // Adjust the factor for desired line length




            // End
            let mut currentEndX : f32 = end_x;
            let mut currentEndY : f32 = end_y;

            for i2 in 0..rects.len() {

                // Box End
                let (ex, ey) = line_rect_intersection(x, y, end_x, end_y, &rects[i2].rect, size);

                if find_line_length(x, y, ex, ey) < find_line_length(x, y, currentEndX, currentEndY) {
                    currentEndX = ex;
                    currentEndY = ey;
                }

            }

            vertices[i] = Vector2 { x: currentEndX, y: currentEndY };



            //draw_handle.draw_line(x as i32, y as i32, currentEndX as i32, currentEndY as i32, Color::GREEN);
        }


        // Connect
        for i in 1..vertices.len() {
            draw_handle.draw_line(vertices[i - 1].x as i32, vertices[i - 1].y as i32, vertices[i].x as i32, vertices[i].y as i32, Color::RED);
        }
        draw_handle.draw_line(vertices[vertices.len()-1].x as i32, vertices[vertices.len()-1].y as i32, vertices[0].x as i32, vertices[0].y as i32, Color::RED);

        //let c_string = CString::new("./src/red.png").expect("CString conversion failed");


        //let texture: Texture = unsafe { LoadTexture(c_string.as_ptr()) };
        //let custom_texture = CustomTexture { texture };

        //draw_handle.draw_texture_poly(custom_texture, self.pos, &vertices, &vertices, Color::RED);

    }

    fn update(&mut self) {
        self.pos.x = unsafe { GetMouseX() } as f32;
        self.pos.y = unsafe {
            GetMouseY()
        } as f32;
    }


}


fn line_rect_intersection(start_x: f32, start_y: f32, end_x: f32, end_y: f32, rect: &Rectangle, max_size : f32) -> (f32, f32) {

    // Only Check Close
    const max: f32 = ((60 * 60) + (60 * 60)) as f32;
    let difx = rect.x - start_x;
    let dify = rect.y - start_y;
    let mag = difx * difx + dify * dify;
    if mag.sqrt() > max_size - 1f32 + max.sqrt() {
        return (end_x,end_y)
    }
    // end

    let mut t_min: f32 = 0.0;
    let mut t_max: f32 = 1.0;

    let dx = end_x - start_x;
    let dy = end_y - start_y;

    let p1 = [-dx, dx, -dy, dy];
    let p2 = [start_x - rect.x, rect.x + rect.width - start_x, start_y - rect.y, rect.y + rect.height - start_y];

    for i in 0..4 {
        if p1[i] == 0.0 {
            if p2[i] < 0.0 {
                return (end_x, end_y); // Line is outside the rectangle, return the original end point
            }
        } else {
            let t = p2[i] / p1[i];
            if p1[i] < 0.0 {
                t_min = t_min.max(t);
            } else {
                t_max = t_max.min(t);
            }
        }
    }

    if t_min > t_max {
        return (end_x, end_y ); // Line is outside the rectangle, return the original end point
    }

    let collision_x = start_x + t_min * dx;
    let collision_y = start_y + t_min * dy;
    return (collision_x, collision_y);
}


struct Box {
    rect : Rectangle
}

impl Box {

    fn new (xPosition: f32, yPosition: f32) -> Self {
        Self {
            rect: Rectangle { x: xPosition, y: yPosition, width: 60f32, height: 60f32 }
        }
    }

    fn draw(&self, drawHandle: &mut RaylibDrawHandle) {
        drawHandle.draw_rectangle(self.rect.x as i32, self.rect.y as i32, self.rect.width as i32, self.rect.height as i32, Color::PURPLE);
    }

}




struct Tilemap {
    data: Vec<Box>
}

impl Tilemap {
    fn new ( map: [[&str; 32]; 18]) -> Self {

        let mut data_final: Vec<Box> = Vec::new();

        for row in 0..map.len() {
            for column in 0..map[row].len() {
                if map[row][column] == "1" {
                    let boxx: Box = Box::new(60f32 * column as f32, 60f32 * row as f32);
                    data_final.push(boxx);
                }
            }
        }


        

        return Tilemap { data: data_final }
    }

    fn draw (&self, drawHandle: &mut RaylibDrawHandle) {
        for item_number in 0..self.data.len() {
            self.data[item_number].draw(drawHandle)
        }
    }
}







fn main() {

    let world = [
        ["1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "1", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "1", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1"],
        ["1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1"]
    ];

    let tilemap = Tilemap::new(world);

    let mut player = Player::new(500f32);

    let ( mut rl, thread ) = raylib::init().size(1920, 1080).title("Title").build();

    while !rl.window_should_close() {
        let mut d = rl.begin_drawing(&thread);

        d.clear_background(Color::DARKGRAY);

        player.update();
        player.draw(&mut d);

        

        tilemap.draw(&mut d);
        player.draw_points(&mut d, &tilemap.data);
        d.draw_fps(12, 12);


    }


}
